
import 'package:dio/dio.dart';
import 'package:fawaterk_payment/data/fawaterk_payment_intent_payload.dart';
import 'package:fawaterk_payment/data/payment_method.dart';
import 'package:fawaterk_payment/response/fawaterk_response.dart';
import 'package:fawaterk_payment/response/fawaterk_response_interface.dart';

class FawaterkPayment {

  Dio dio = Dio();

  final String apiUrl;
  final String apiKey;

  FawaterkPayment({
    required this.apiUrl,
    required this.apiKey,
  });

  Future<List<PaymentMethod>> getMethods() async {
    List<PaymentMethod> methods = <PaymentMethod> [];

    try {
      Response response = await dio.get(
        '$apiUrl/api/v2/getPaymentmethods',
        options: Options(
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer $apiKey'
          }
        )
      );

      for (Map<String, dynamic> element in response.data['data']) {
        methods.add(PaymentMethod.fromJson(element));
      }
    } catch (e) {
      //
    }

    return methods;
  }

  Future<FawaterkResponseInterface?> submitPayment(
    FawaterkPaymentIntentPayload payload, {
      void Function()? onSuccess,
      void Function()? onFailure,
    }
  ) async {

    dio.interceptors.add(
      InterceptorsWrapper(
        onRequest: (RequestOptions options, RequestInterceptorHandler handler) {
          return handler.next(options);
        },
        onResponse: (Response response, ResponseInterceptorHandler handler) {
          return handler.next(response);
        },
        onError: (DioException e, ErrorInterceptorHandler handler) {
          
          // Do something with response error.
          // If you want to resolve the request with some custom data,
          // you can resolve a `Response` object using `handler.resolve(response)`.
          return handler.next(e);
        },
      ),
    );

    try {
      Response response = await dio.post(
        '$apiUrl/api/v2/invoiceInitPay',
        data: {
          "payment_method_id": payload.methodId,
          "cartTotal": payload.cartTotal,
          "currency": payload.currency,
          "customer": {
            "first_name": payload.customer.firstName,
            "last_name": payload.customer.lastName,
            "email": payload.customer.email,
            "phone": payload.customer.phoneNumber,
            "address": payload.customer.address,
          },
          "redirectionUrls": {
            "successUrl": payload.redirectUrls.success,
            "failUrl": payload.redirectUrls.fail,
            "pendingUrl": payload.redirectUrls.pending,
          },
          "cartItems": payload.cart.items.map((item) {
            return {
              "name": item.name,
              "price": item.price,
              "quantity": item.quantity,
            };
          }).toList()
        },
        options: Options(
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer $apiKey'
          }
        )
      );

      return FawaterkResponse(
        response: response,
        // methodId: payload.methodId
      ).respond();
    } catch (e) {
      return FawaterkResponse(
        response: null,
        // methodId: payload.methodId
      ).respond();
    }
  }
}
